#!/usr/bin/python3
# -*- coding: utf-8 -*-

import sys
import json
from collections import defaultdict

senses = ['Staat', 'Kind'] # List der Lesarten
ambiguous_word = ':'.join(senses)   # zu desambiguierendes Wort
relevant_tags = set(['ADJA', 'ADJD', 'ADV', 'NE', 'NN', 'VVFIN', 'VVINF', 'VVPP', 'VVIZU'])
max_dist = 50

if len(sys.argv) != 3:
    sys.exit('Usage: test.py model-file input-file')

# Modell aus Datei lesen
with open(sys.argv[1]) as file:
    logprob = json.load( file )

# zu desambiguierende Daten einlesen
with open(sys.argv[2]) as file:
    data = [line.strip().split('\t') for line in file]
    _, tags, lemmas = zip(*data)

# Vorkommen ambiger Wörter bestimmen
for i in range(len(lemmas)):
    if lemmas[i] == ambiguous_word:
        # Kontextwörter extrahieren
        start = max(0, i-max_dist)
        end = min(len(tags), i+max_dist+1)
        logp = {sense:0.0 for sense in senses}
        for k in range(start, end):
            # logarithmierte Wahrscheinlichkeiten aller
            # Kontextwörter aufsummieren
            if tags[k] in relevant_tags and k != i:
                for s in senses:
                    if lemmas[k] in logprob[s]:
                        logp[s] += logprob[s][lemmas[k]] 

        # argmax-Operation berechnen
        best_sense = max(logp.keys(), key=logp.get)

        # Position und beste Lesart ausgeben
        print('Zeile %d: %s' % (i, best_sense))
