#!/usr/bin/python3
# -*- coding: utf-8 -*-

import sys
import math
import json
from collections import defaultdict

merged_words = ['Staat', 'Kind']
relevant_tags = set(['ADJA', 'ADJD', 'ADV', 'NE', 'NN', 'VVFIN', 'VVINF', 'VVPP', 'VVIZU'])
max_dist = 50

if len(sys.argv) != 3:
    sys.exit('Usage: train.py data-file model-file')

# Trainingsdaten einlesen
print("reading data...", file=sys.stderr)
with open(sys.argv[1]) as file:
    lemmas = []
    tags = []
    for line in file:
        word, tag, lemma = line.strip().split('\t')
        lemmas.append(lemma)
        tags.append(tag)

print("done", file=sys.stderr)
print("estimating probabilities...", file=sys.stderr)

# Häufigkeiten zählen
count  = defaultdict(lambda: defaultdict(int))
for i in range( len(tags) ):
    # Ist das aktuelle Wort im Pseudowort enthalten?
    if lemmas[i] in merged_words:
        # Kontextwörter extrahieren
        start = max(0, i-max_dist)
        end = min(len(tags), i+max_dist+1)
        # über alle Wörter bis Maximalabstand max_dist iterieren
        for k in range(start, end):
            # falls das Kontextwort ein relevantes Tag hat und
            # nicht mit dem aktuellem Wort identisch ist
            if tags[k] in relevant_tags and k != i:
                # Häufigkeit des Paares erhöhen
                count[lemmas[i]][lemmas[k]] += 1

# Gesamthäufigkeiten der Bedeutungen und Kontextwörter berechnen
sense_count = defaultdict(int)
cword_count = defaultdict(int)
for sense in count:
    for word, freq in count[sense].items():
        sense_count[sense] += freq
        cword_count[word] += freq

# Wahrscheinlichkeiten schätzen (Witten-Bell-Methode)
N = sum(sense_count.values())  # Gesamtzahl der Kontextwörter
logprob = {}  # logarithmierte Wahrscheinlichkeiten der Kontextwörter
for sense in count:  # für alle Lesarten
    logprob[sense] = {}
    alpha = len(count[sense])  # Zahl der Types bei den Kontextwörtern von Lesart s
    for word, freq in cword_count.items():  # für alle Kontextwörter (insgesamt)
        # Apriori-Wortwahrscheinlichkeit des Wortes berechnen
        pp = freq / N
        # mit Witten-Bell geglättete Wahrscheinlichkeit berechnen
        p = (count[sense].get(word, 0.0) + alpha * pp)/(sense_count[sense] + alpha)
        logprob[sense][word] = math.log(p)
print("done", file=sys.stderr)

# Modell in einer Datei speichern
with open(sys.argv[2],'wt') as file:
    json.dump(logprob, file)
